import { HttpClient } from '@angular/common/http';
import { Component, Inject, OnInit } from '@angular/core';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material/dialog';
import { ToastrService } from 'ngx-toastr';

@Component({
  selector: 'app-create-subnet',
  templateUrl: './create-subnet.component.html',
  styleUrls: ['./create-subnet.component.css']
})
export class CreateSubnetComponent implements OnInit {
  aulasForm: FormGroup;
  subnetForm: FormGroup;

  aulas: any[] = []; // Lista de aulas
  selectedAulas: any[] = []; // Aulas seleccionadas
  nextServerOptions: string[] = []; // Opciones de Next Server
  bootFileNameOptions: string[] = []; // Opciones de Boot File Name
  isEditMode: boolean = false;

  constructor(
    private fb: FormBuilder, // FormBuilder para manejar los formularios
    private toastService: ToastrService,
    private http: HttpClient,
    public dialogRef: MatDialogRef<CreateSubnetComponent>,
    @Inject(MAT_DIALOG_DATA) public data: any  
  ) { 
    // Inicializar los formularios
    this.aulasForm = this.fb.group({
      selectedAulas: [[], Validators.required]
    });

    this.subnetForm = this.fb.group({
      name: ['', Validators.required],
      netmask: ['', Validators.required],
      ipAddress: ['', Validators.required],
      nextServer: ['', Validators.required],
      bootFileName: ['', Validators.required]
    });
  }

  ngOnInit(): void {
    this.loadAulas(); // Cargar las aulas al iniciar el componente

    if (this.data && this.data.subnet) {
      this.isEditMode = true;
      this.subnetForm.patchValue({
        name: this.data.subnet.name,
        netmask: this.data.subnet.netmask,
        ipAddress: this.data.subnet.ipAddress,
        nextServer: this.data.subnet.nextServer,
        bootFileName: this.data.subnet.bootFileName
      });
      this.selectedAulas = this.data.subnet.selectedAulas || [];
      this.updateServerAndBootFileOptions();
    }
  }

  loadAulas(): void {
    this.http.get<any>('http://127.0.0.1:8001/organizational-units?page=1&itemsPerPage=30')
      .subscribe(response => {
        this.aulas = response['hydra:member'];
      }, error => {
        console.error('Error loading aulas:', error);
        this.toastService.error('Error al cargar las aulas');
      });
  }

  onAulasSelectionChange(event: any): void {
    this.selectedAulas = event.value;
    this.updateServerAndBootFileOptions();
  }

  updateServerAndBootFileOptions(): void {
    // Vaciar las opciones previas
    this.nextServerOptions = [];
    this.bootFileNameOptions = [];

    // Agregar las opciones de "nextServer" y "bootFileName" basadas en las aulas seleccionadas
    this.selectedAulas.forEach(aula => {
      if (aula.networkSettings) {
        if (aula.networkSettings.nextServer && !this.nextServerOptions.includes(aula.networkSettings.nextServer)) {
          this.nextServerOptions.push(aula.networkSettings.nextServer);
        }
        if (aula.networkSettings.bootFileName && !this.bootFileNameOptions.includes(aula.networkSettings.bootFileName)) {
          this.bootFileNameOptions.push(aula.networkSettings.bootFileName);
        }
      }
    });
  }

  onNoClick(): void {
    this.dialogRef.close();
  }

  submitForm(): void {
    if (this.aulasForm.invalid || this.subnetForm.invalid) {
      this.toastService.error('Por favor, complete los formularios.');
      return;
    }

    const payload = {
      ...this.subnetForm.value,
      /* selectedAulas: this.selectedAulas.map(aula => aula.uuid)  */// Enviar solo los UUID de las aulas seleccionadas
    };

    if (this.isEditMode) {
      this.http.patch(`http://127.0.0.1:8001/subnets/${this.data.subnet.uuid}`, payload)
        .subscribe({
          next: (response) => {
            console.log('Success:', response);
            this.toastService.success('Configuración de red actualizada exitosamente');
            this.dialogRef.close();
          },
          error: (error) => {
            console.error('Error:', error);
            this.toastService.error('Error al actualizar la configuración de red');
          }
        });
    } else {
      
      this.http.post('http://127.0.0.1:8001/subnets', payload)
        .subscribe({
          next: (response) => {
            console.log('Success:', response);
            this.toastService.success('Configuración de red añadida exitosamente');
            this.dialogRef.close();
          },
          error: (error) => {
            console.error('Error:', error);
            this.toastService.error('Error al añadir la configuración de red');
          }
        }); 
    }
  }
}
